local L = LibStub("AceLocale-3.0"):GetLocale("Talented")
local Talented = Talented

local LAYOUT_BASE_X = 4
local LAYOUT_BASE_Y = 24
-- local LAYOUT_MAX_COLUMNS = 4
-- local LAYOUT_MAX_ROWS = 9

local LAYOUT_OFFSET_X, LAYOUT_OFFSET_Y, LAYOUT_DELTA_X, LAYOUT_DELTA_Y
local LAYOUT_SIZE_X, LAYOUT_SIZE_Y, LAYOUT_SIZE_Y_PET

local function RecalcLayout(offset)
	if LAYOUT_OFFSET_X ~= offset then

		LAYOUT_OFFSET_X = offset
		LAYOUT_OFFSET_Y = LAYOUT_OFFSET_X

		LAYOUT_DELTA_X = LAYOUT_OFFSET_X / 2
		LAYOUT_DELTA_Y = LAYOUT_OFFSET_Y / 2

		LAYOUT_SIZE_X = --[[LAYOUT_MAX_COLUMNS]] 4 * LAYOUT_OFFSET_X + LAYOUT_DELTA_X
		LAYOUT_SIZE_Y = --[[LAYOUT_MAX_ROWS]] 11 * LAYOUT_OFFSET_Y + LAYOUT_DELTA_Y
		LAYOUT_SIZE_Y_PET = --[[LAYOUT_MAX_ROWS]] 7 * LAYOUT_OFFSET_Y + LAYOUT_DELTA_Y

		return true
	end
end

local function offset(row, column)
	return (column - 1) * LAYOUT_OFFSET_X + LAYOUT_DELTA_X, - ((row - 1) * LAYOUT_OFFSET_Y + LAYOUT_DELTA_Y)
end

local TalentView = {}
Talented.views = {}
function TalentView:init(frame, type)
	table.insert(Talented.views, self)
	self.base = frame
	self.type = type
	self.maxpoints = GetUnspentTalentPoints(nil, type == "pet")
	self.elements = {}
end

function TalentView:SetUIElement(element, ...)
	self.elements[strjoin("-", ...)] = element
end

function TalentView:GetUIElement(...)
	return self.elements[strjoin("-", ...)]
end

function TalentView:SaveFramePosition()
	local p, _ = Talented.db.profile.positions[self.type]
	p.point, _, p.relativePoint, p.offsetx, p.offsety = self.base:GetPoint()
	self.base:SetPoint(p.point, UIParent, p.relativePoint, p.offsetx, p.offsety)
end

function TalentView:SetViewMode(mode, force)
	if mode ~= self.mode or force then
		self.mode = mode
		self:Update()
	end
end

function TalentView:SetViewFlags(spec)
	local update
	if spec ~= self.spec then
		self.spec = spec
		update = true
	end

	if update then
		self:Update()
	end
end

function TalentView:SetClass(class, force)
	if self.class == class and not force then return end
	Talented.Pool:changeSet(self.type)
	local talents = Talented:UncompressSpellData(class)

	for tab, tree in ipairs(talents) do
		local frame = Talented:MakeTalentFrame(self.base, LAYOUT_SIZE_X, self.type == "pet" and LAYOUT_SIZE_Y_PET or LAYOUT_SIZE_Y)
		frame.tab = tab
		frame.view = self
		frame.pet = (self.type == "pet")

		local background = Talented.tabdata[class][tab].background
		frame.topleft:SetTexture("Interface\\TalentFrame\\"..background.."-TopLeft")
		frame.topright:SetTexture("Interface\\TalentFrame\\"..background.."-TopRight")
		frame.bottomleft:SetTexture("Interface\\TalentFrame\\"..background.."-BottomLeft")
		frame.bottomright:SetTexture("Interface\\TalentFrame\\"..background.."-BottomRight")

		self:SetUIElement(frame, tab)

		for idx, talent in ipairs(tree) do
			local button = Talented:MakeButton(frame)
			button.id = idx

			self:SetUIElement(button, tab, idx)

			button:SetPoint("TOPLEFT", offset(talent.row, talent.column))
			button.texture:SetTexture(Talented:GetTalentIcon(class, tab, idx))
			button:Show()
		end

		for idx, talent in ipairs(tree) do
			local req = talent.req
			if req then
				local elements = {}
				Talented.DrawLine(elements, frame, offset, talent.row, talent.column, tree[req].row, tree[req].column)
				self:SetUIElement(elements, tab, idx, req)
			end
		end

		frame:SetPoint("TOPLEFT", (tab-1) * LAYOUT_SIZE_X + LAYOUT_BASE_X, -LAYOUT_BASE_Y)
	end
	self.base:SetWidth(#talents * LAYOUT_SIZE_X + LAYOUT_BASE_X * 2)
	self.base:SetHeight(LAYOUT_SIZE_Y + LAYOUT_BASE_Y + LAYOUT_BASE_X)
	self.base:SetScale(Talented.db.profile.scale)

	self.class = class
	self:Update()
end

function TalentView:SetTemplate(template, target)
	self.template, self.target = template, target
	self:SetClass(template.class)
	return self:Update()
end

function TalentView:Update()
	local template, target = self.template, self.target
	local total = 0
	local info = Talented:UncompressSpellData(template.class)
	local at_cap = Talented:IsTemplateAtCap(template)
	for tab, tree in ipairs(info) do
		local count = 0
		for index, talent in ipairs(tree) do
			local rank = template[tab][index]
			count = count + rank
			local button = self:GetUIElement(tab, index)
			local color = GRAY_FONT_COLOR
			local state = Talented:GetTalentState(template, tab, index)
			if state == "empty" and (at_cap or self.mode == "view") then
				state = "unavailable"
			end
			if state == "unavailable" then
				button.texture:SetDesaturated(1)
				button.slot:SetVertexColor(0.65, 0.65, 0.65)
				button.rank:Hide()
				button.rank.texture:Hide()
			else
				button.rank:Show()
				button.rank.texture:Show()
				button.rank:SetText(rank)
				button.texture:SetDesaturated(0)
				if state == "full" then
					color = NORMAL_FONT_COLOR
				else
					color = GREEN_FONT_COLOR
				end
				button.slot:SetVertexColor(color.r, color.g, color.b)
				button.rank:SetVertexColor(color.r, color.g, color.b)
			end
			local req = talent.req
			if req then
				local ecolor = color
				if ecolor == GREEN_FONT_COLOR then
					if self.mode == "edit" then
						local s = Talented:GetTalentState(template, tab, req)
						if s ~= "full" then
							ecolor = RED_FONT_COLOR
						end
					else
						ecolor = NORMAL_FONT_COLOR
					end
				end
				for _, element in ipairs(self:GetUIElement(tab, index, req)) do
					element:SetVertexColor(ecolor.r, ecolor.g, ecolor.b)
				end
			end
			local targetvalue = target and target[tab][index]
			if targetvalue and (targetvalue > 0 or rank > 0) then
				local btarget = Talented:GetButtonTarget(button)
				btarget:Show()
				btarget.texture:Show()
				btarget:SetText(targetvalue)
				local tcolor
				if rank < targetvalue then
					tcolor = LIGHTBLUE_FONT_COLOR
				elseif rank == targetvalue then
					tcolor = GRAY_FONT_COLOR
				else
					tcolor = RED_FONT_COLOR
				end
				btarget:SetVertexColor(tcolor.r, tcolor.g, tcolor.b)
			elseif button.target then
				button.target:Hide()
				button.target.texture:Hide()
			end
		end
		local frame = self:GetUIElement(tab)
		frame.name:SetFormattedText(L["%s (%d)"], Talented.tabdata[template.class][tab].name, count)
		total = total + count
		local clear = frame.clear
		if self.mode ~= "edit" or count <= 0 or self.spec then
			clear:Hide()
		else
			clear:Show()
		end
	end
	local points = self.base.points
	if points then
		local maxpoints = self.maxpoints
		if Talented.db.profile.show_level_req then
			local level_req = (total == 0) and 1  or (9 + total)
			points:SetFormattedText(L["Level %d"], level_req)
		else
			points:SetFormattedText(L["%d/%d"], total, maxpoints)
		end
		local color
		if total < maxpoints then
			color = GREEN_FONT_COLOR
		elseif total > maxpoints then
			color = RED_FONT_COLOR
		else
			color = NORMAL_FONT_COLOR
		end
		points:SetTextColor(color.r, color.g, color.b)
	end
	local pointsleft = self.base.pointsleft
	if pointsleft then
		if maxpoints ~= total and template.talentGroup then
			pointsleft:Show()
			pointsleft.text:SetFormattedText(L["You have %d talent points left"], maxpoints - total)
		else
			pointsleft:Hide()
		end
	end
end

function TalentView:SetTooltipInfo(owner, tab, index)
	Talented:SetTooltipInfo(owner, tab, index, self.type == "pet")
end

function TalentView:OnTalentClick(button, tab, index)
	if IsModifiedClick"CHATLINK" then
		local link = Talented:GetTalentLink(self.template, tab, index)
		if link then
			ChatEdit_InsertLink(link)
		end
	else
		self:UpdateTalent(tab, index, button == "LeftButton" and 1 or -1)
	end
end

function Talented:UpdateTalent(tab, index, offset)
	if self.mode ~= "edit" then return end
	if self.template == Talented.current then
		-- Applying talent
		if offset > 0 then
			Talented:LearnTalent(tab, index, self.type == "pet")
		end
		return
	end
	local template = self.template

	if offset > 0 and self:IsTemplateAtCap(template) then return end
	local s = Talented:GetTalentState(template, tab, index)

	local ranks = Talented:GetTalentRanks(template.class, tab, index)
	local original = template[tab][index]
	local value = original + offset
	if value < 0 or s == "unavailable" then
		value = 0
	elseif value > ranks then
		value = ranks
	end
	Talented:Debug("Updating %d-%d : %d -> %d (%d)", tab, index, original, value, offset)
	if value == original or not Talented:ValidateTalentBranch(template, tab, index, value) then
		return
	end
	template[tab][index] = value
	self:Update()
	Talented:UpdateTooltip()
	return true
end

function TalentView:ClearTalentTab(tab)
	local template = self.template
	if template and not template.talentGroup then
		local tab = template[tab]
		for index, value in ipairs(tab) do
			tab[index] = 0
		end
	end
	self:Update()
end

Talented.TalentView = {
	__index = TalentView,
	new = function (self, ...)
		local view = setmetatable({}, self)
		view:init(...)
		return view
	end,
}

function Talented:ReLayout()
	if RecalcLayout(self.db.profile.offset) then
		for _, view in ipairs(self.views) do
			view:SetClass(view.class, true)
		end
	end
	for _, view in ipairs(self.views) do
		view.base:SetScale(self.db.profile.scale)
	end
end

function Talented:PetTest()
	RecalcLayout(self.db.profile.offset)
	self:PET_TALENT_POINTS_CHANGED()
	local f = CreateFrame("Frame", nil, UIParent)
	local v = self.TalentView:new(f, "pet")
	v:SetTemplate(self.pet_current)
	f:SetPoint"CENTER"
	f:Show()
end
